/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.wiki.fileaccess;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Properties;

import javax.activation.MimetypesFileTypeMap;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.logging.Logger;
import org.jboss.wiki.FileMediaDataSource;
import org.jboss.wiki.ServletWikiSession;
import org.jboss.wiki.SimpleCredentials;
import org.jboss.wiki.WikiAttachment;
import org.jboss.wiki.WikiCommon;
import org.jboss.wiki.WikiContext;
import org.jboss.wiki.WikiEngine;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;
import org.jboss.wiki.exceptions.WikiManagementNotFoundException;
import org.jboss.wiki.management.WikiServiceManagement;


/**
 * A filter that enables to download files form the jspwiki structure +
 * redirects to pages by typing http://domain/wiki/Page
 * 
 * @author tomek
 */

public class WikiFileAccessFilter implements Filter {
	/**
	 * <code>BUF_SIZE</code> - size of a buffer, which is used while reading a
	 * file and sending it to the client.
	 */

	private final int BUF_SIZE = 32768;

	private String attachementsPath;

	private String wikiHome;

	private WikiEngine wikiEngine;

	/*
	 * private ForgeHelper forgeHelper; private ContentManager contentManager;
	 */
	private MimetypesFileTypeMap mimeTypes;

	private Logger log;

	public void init(FilterConfig conf) {
		// get wikiEnigne form wikiMenagement

		log = Logger.getLogger(WikiFileAccessFilter.class);

		try {
			WikiServiceManagement wikiServiceMenagement = WikiCommon
					.getWikiManagement();
			wikiEngine = wikiServiceMenagement.getWikiEngine();
		} catch (WikiManagementNotFoundException e1) {
			log.error("Couldn't find WikiManagement", e1);
			wikiEngine = new WikiEngine();
		}

		// load mime types map

		mimeTypes = new MimetypesFileTypeMap(WikiFileAccessFilter.class
				.getResourceAsStream("mime.types"));

		// load path to attachements from filedatasource.properties

		Properties props = new Properties();

		try {
			props.load(FileMediaDataSource.class
					.getResourceAsStream("filedatasource.properties"));

			// get the path to attachment property
			attachementsPath = props.getProperty("pathToAttachements");

			props.load(WikiPlugin.class
					.getResourceAsStream("wikiPlugins.properties"));

			// get the host
			wikiHome = props.getProperty("wikiHome");

		} catch (IOException e) {
			log.error(e);
		}

		log.info("Loaded attachements path: " + attachementsPath);
	}

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		String langCode = wikiEngine.defaultLangugeCode;

		if (request instanceof HttpServletRequest) {
			HttpServletRequest httpRequest = (HttpServletRequest) request;
			HttpServletResponse httpResponse = (HttpServletResponse) response;

			WikiSession wikiSession = new ServletWikiSession(httpRequest
					.getSession());

			String hostURL = ((request.isSecure()) ? "https" : "http")
					+ "://"
					+ request.getServerName()
					+ ((request.getServerPort() == 80) ? "" : ":"
							+ request.getServerPort());

			wikiSession.setAttribute("hostURL", hostURL);

			String requestURI = URLDecoder.decode(httpRequest.getRequestURI(),
					"UTF-8");

			// get off "/wiki/" from the begining
			requestURI = requestURI.substring("/wiki/".length());

			// support old type link with Wiki.jsp?page=
			if (requestURI.equals("Wiki.jsp")) {
				if (httpRequest.getParameter("page") != null)
					httpResponse.sendRedirect(hostURL + wikiHome + "&page="
							+ httpRequest.getParameter("page"));
				else
					httpResponse.sendRedirect(hostURL + wikiHome);

				return;
			}

			while (requestURI.endsWith("/")) {
				// get rid of ending "/" character(s)
				requestURI = requestURI.substring(0, requestURI.length() - 1);
			}

			int version = -1;

			if (requestURI.matches(".*:\\d+")) {
				// there is a ":" at the end of request uri - pick a version

				int verStart = requestURI.lastIndexOf(":");
				version = Integer.valueOf(requestURI.substring(verStart + 1));
				requestURI = requestURI.substring(0, verStart);
			}
			// Parse URI to get filename and page name
			String[] tokens = requestURI.split("[/]");

			if ((tokens.length == 1) && (tokens[0].equals(""))) {
				// show Main page - as there is no place to go specified :)
				httpResponse.sendRedirect(hostURL + wikiHome);
				return;
			}

			// check if user is browsing different then default language
			if (wikiEngine.getUsedLanguageCodes().containsKey(
					tokens[0].toUpperCase())) {
				langCode = tokens[0].toUpperCase();
				// redo tokens but without language
				if (requestURI.startsWith(tokens[0] + "/")) {
					requestURI = requestURI.substring((tokens[0] + "/")
							.length());
				} else {
					requestURI = requestURI.substring((tokens[0]).length());
				}

				tokens = requestURI.split("[/]");

				if ((tokens.length == 1) && (tokens[0].equals(""))) {
					// show Main page - as there is no place to go specified :)
					httpResponse.sendRedirect(hostURL + wikiHome + "&language="
							+ langCode);
					return;
				}
			}

			// check if all tokens are UpperCase (meanins they are all wiki
			// pages)
			boolean isAllUpperCase = true;

			for (int i = 0; i < tokens.length; i++)
				// uppercase or UTF character
				if (!(Character.isUpperCase(tokens[i].charAt(0)) || (tokens[i]
						.charAt(0) > 127 && Character.isLetter(tokens[i]
						.charAt(0))))) {
					isAllUpperCase = false;
					break;
				}

			if (isAllUpperCase) {
				String pageName = "";

				for (int i = 0; i < tokens.length; i++) {
					pageName += "/" + URLEncoder.encode(tokens[i], "UTF-8");
				}

				if (Character.isUpperCase(tokens[tokens.length - 1].charAt(0))
						|| (tokens[tokens.length - 1].charAt(0) > 127 && Character
								.isLetter(tokens[tokens.length - 1].charAt(0)))) {
					httpResponse.sendRedirect(hostURL + wikiHome + "&page="
							+ pageName.substring(1)
							+ ((version != -1) ? "&version=" + version : "")
							+ "&language=" + langCode);
				} else {
					PrintWriter out = httpResponse.getWriter();
					httpResponse.setContentType("text/html");
					out
							.println("<html><body><h3>ERROR</h3><br />\nPage name must start with upper case letter.</body></html>");
				}

			} else if (tokens[0].equals(WikiCommon.wikiButtons)) {
				// show apropriate buttons
				InputStream imageIS = WikiFileAccessFilter.class
						.getResourceAsStream("/images/"
								+ tokens[tokens.length - 1]);

				ServletOutputStream os = response.getOutputStream();

				response.setContentType(mimeTypes
						.getContentType(tokens[tokens.length - 1]));

				// Transferring the bytes
				try {
					byte[] buffer = new byte[BUF_SIZE];
					int read;
					while ((read = imageIS.read(buffer)) != -1)
						os.write(buffer, 0, read);
				} catch (Exception e2) {
					// Nothing that we can really do. Just send an
					// incomplete
					// file.
				} finally {
					os.close();
					imageIS.close();
				}

			} else if ((Character.isLowerCase(tokens[0].charAt(0)))
					&& (tokens.length > 1)) {
				boolean isPageNameUC = true;

				for (int i = 1; i < tokens.length; i++)
					if (Character.isLowerCase(tokens[i].charAt(0))) {
						isPageNameUC = false;
						break;
					}

				if (isPageNameUC) {
					// SHOW wikiType firstToken with page secondToken

					String wikiTypeName = tokens[0];

					if (wikiEngine.wikiTypeExists(wikiTypeName)) {

						// get pageName
						String pageName = "";
						for (int i = 1; i < tokens.length; i++) {
							pageName += "/" + tokens[i];
						}

						pageName = pageName.substring(1);

						WikiPage page = wikiEngine.getByName(pageName,
								new WikiContext(new SimpleCredentials(""),
										wikiEngine.getWikiType(wikiTypeName),
										wikiSession), langCode);

						httpResponse.setContentType(wikiEngine.getWikiType(
								wikiTypeName).getContentType());

						httpResponse.setContentLength(page.getPageContent()
								.length());

						httpResponse.getWriter().print(page.getPageContent());

						httpResponse.getWriter().flush();
					} else {
						// show error response
						httpResponse.setContentType("text/html");
						httpResponse.getWriter().println(
								"<html><body><h3>ERROR</h3><br />\nThere is no such wikiType: "
										+ wikiTypeName + "</body></html>");
					}
				} else {
					httpResponse.setContentType("text/html");
					httpResponse
							.getWriter()
							.println(
									"<html><body><h3>ERROR</h3><br />\nPage name must start with upper case letter.</body></html>");
				}
			} else if (tokens[tokens.length - 1].matches("file:.*")) {
				// Show file of a page

				// get pageName
				String pageName = "";
				for (int i = 0; i < tokens.length - 1; i++) {
					pageName += ((i > 0) ? "/" : "") + tokens[i];
				}

				String fileName = tokens[tokens.length - 1].substring("file:"
						.length());

				WikiAttachment wikiAttachment;

				if (version == -1) {
					wikiAttachment = wikiEngine.getAttachment(pageName,
							fileName, langCode);
				} else {
					wikiAttachment = wikiEngine.getAttachment(pageName,
							fileName, version, langCode);
				}

				if (wikiAttachment != null) {
					InputStream fileIS = wikiAttachment.getInputStream();

					ServletOutputStream os = response.getOutputStream();

					response.setContentType(mimeTypes.getContentType(fileName));
					// response.setContentLength((int) fileIS.);

					// Transferring the bytes
					try {
						byte[] buffer = new byte[BUF_SIZE];
						int read;
						while ((read = fileIS.read(buffer)) != -1)
							os.write(buffer, 0, read);
					} catch (Exception e2) {
						// Nothing that we can really do. Just send an
						// incomplete
						// file.
					} finally {
						os.close();
						fileIS.close();
					}
				} else {
					ServletOutputStream os = response.getOutputStream();

					response.setContentType("text/html");

					os
							.println("<html><body><h3>ERROR</h3><br />\nThere is no such file or there is no such version of file</body></html>");
				}
			} else {
				ServletOutputStream os = response.getOutputStream();

				response.setContentType("text/html");

				os
						.println("<html><body><h3>ERROR</h3><br />\nThere is lack of page name or page name should be uppercase</body></html>");
			}
		} else {
			throw new IOException(
					"This servlet should be called as HTTPServletRequest");
		}
	}

	public void destroy() {

	}
}
