/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import org.jboss.logging.Logger;

public class FileAttachmentDataSource implements AttachmentDataSource{

	private final static String OTHER_LANG_DIR = "LANG/";
	
	private String pathToAttachments;
	
	private String pathToAttTrash;
	
	private final int BUF_SIZE = 32768;
	
	private Logger log = Logger.getLogger(this.getClass());
	
	private String langCode;
	
	public FileAttachmentDataSource() {
		init("");
	}
	
	private void init(String prefix) {
		log = Logger.getLogger(FileMediaDataSource.class);

		log.info("Looking for props file !");

		Properties fileDSProps = new Properties();

		try {

			fileDSProps.load(FileMediaDataSource.class
					.getResourceAsStream(FileMediaDataSource.propFileName));

		} catch (IOException ioe) {
			log.error("Can't load the file " + FileMediaDataSource.propFileName + "\n", ioe);
		}

		loadProperties(fileDSProps, prefix);

		File attWikiDir = new File(pathToAttachments);

		if (!attWikiDir.exists())
			attWikiDir.mkdirs();

		// create trash directory for deleted attachments if it doesn't exists

		pathToAttTrash = pathToAttachments + "/" + AttachmentDataSource.attTrashName;

		File pathToAttachmentsFile = new File(pathToAttTrash);

		if (!pathToAttachmentsFile.exists()) {
			pathToAttachmentsFile.mkdirs();
		}

	}
	
	private void loadProperties(Properties fileDSProps, String prefix) {
		if (fileDSProps.getProperty("pathToMedia") != null) {
			pathToAttachments = fileDSProps.getProperty("pathToAttachements")
					+ (prefix.length() > 0 ? "/" + OTHER_LANG_DIR + prefix : "");
			log.info("Path to attachements is: " + pathToAttachments);
			
			File attDir = new File(pathToAttachments);
			
			if (!attDir.exists()) {
				attDir.mkdirs();
				log.info("Dir for attachments created");
			}
		} else {
			System.err
					.println("Your properties file lacks of pathToMedia property. It will not run properly ! Please update "
							+ FileMediaDataSource.propFileName);
		}
	}
	
	public WikiAttachment getAttachment(String pageName, String attachementName) {
		return getAttachment(pageName, attachementName,
				getLastAttachmentVersion(pageName, attachementName));
	}

	public WikiAttachment getAttachment(String pageName,
			String attachementName, int version) {

		String[] tokens = attachementName.split("\\.");

		Properties props = getAttProps(pageName, attachementName, false);

		if (props == null) {
			return null;
		}

		String user = props.getProperty(version + ".author");

		File file = new File(
				pathToAttachments
						+ "/"
						+ pageName
						+ "-att/"
						+ attachementName
						+ "-dir/"
						+ version
						+ "."
						+ ((attachementName.indexOf(".") != -1) ? tokens[tokens.length - 1]
								: "bin"));

		FileInputStream fis;
		try {
			fis = new FileInputStream(file);
			return new WikiAttachment(attachementName, new Date(file
					.lastModified()), user, file.length(), fis, version);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return null;
		}

	}

	public int getLastAttachmentVersion(String pageName, String attachementName) {

		Properties props = getAttProps(pageName, attachementName, false);

		if (props == null)
			return -1;

		// looking for the newest version
		int version = 1;

		while (props.getProperty((version + 1) + ".author") != null) {
			version++;
		}

		return version;
	}

	public long getAttachmentSize(String pageName, String attachementName,
			int version) {
		String[] tokens = attachementName.split("\\.");

		File file = new File(
				pathToAttachments
						+ "/"
						+ pageName
						+ "-att/"
						+ attachementName
						+ "-dir/"
						+ version
						+ "."
						+ ((attachementName.indexOf(".") != -1) ? tokens[tokens.length - 1]
								: "bin"));

		return file.length();
	}

	public void setWikiEngine(WikiEngine wikiEngine) {
		if (langCode == null)
			langCode = wikiEngine.defaultLangugeCode;
	}
	
	public synchronized boolean deleteAttachment(String pageName,
			String attachmentName) {

		String attName = pathToAttachments + "/" + pageName + "-att/"
				+ attachmentName + "-dir";
		String attTrashNameBase = pathToAttTrash + "/" + pageName + "-att/"
				+ attachmentName + "-dir";
		String attTrashName = attTrashNameBase;

		// create dirs to trash
		File trashFileDir = new File(pathToAttTrash + "/" + pageName + "-att/");
		if (!trashFileDir.exists()) {
			trashFileDir.mkdirs();
		}

		// moving file directory
		File attFile = new File(attName);

		if (attFile.exists()) {

			// look if att dir is already in trash (add i-bak)
			File attTrashFile = new File(attTrashName);

			int i = 0;

			if (attTrashFile.exists()) {
				while (attTrashFile.exists()) {
					attTrashName = attTrashNameBase + "-" + (i++) + "-bak";
					attTrashFile = new File(attTrashName);
				}

			}

			if (!attFile.renameTo(attTrashFile)) {
				log.error("[ATTACHMENT DS]:Problems with moving " + attName
						+ " to " + attTrashName);
				return false;
			}

			return true;
		} else {
			log.error("Page " + pageName + " has no " + attachmentName
					+ " attachmnet");
			return false;
		}
	}

	public synchronized boolean deleteAttachments(String pageName) {
		String attName = pathToAttachments + "/" + pageName + "-att";
		String attTrashNameBase = pathToAttTrash + "/" + pageName + "-att";
		String attTrashName = attTrashNameBase;

		// create dirs to trash
		File trashFileDir = new File(pathToAttTrash + "/" + pageName);
		if (!trashFileDir.exists()) {
			trashFileDir.mkdirs();
		}

		// moving file directory
		File attFile = new File(attName);

		if (attFile.exists()) {
			// look if att dir is already in trash (add i-bak)
			File attTrashFile = new File(attTrashName);

			int i = 0;

			if (attTrashFile.exists()) {
				while (attTrashFile.exists()) {
					attTrashName = attTrashNameBase + "-" + (i++) + "-bak";
					attTrashFile = new File(attTrashName);
				}

			}

			if (!attFile.renameTo(new File(attTrashName))) {
				log.error("[ATTACHMENT DS]:Problems with moving " + attName
						+ " to " + attTrashName);
				return false;
			}

			return true;
		} else {
			log.error("Page " + pageName + " has no attachmnets");
			return false;
		}
	}

	public void addLangCode(String langCode, String langDesc) {
		
	}
	
	public void addAttachment(InputStream attFile, String attName, String uid,
			String user) {
		String pageName = uid;
		Properties attProps = getAttProps(pageName, attName, true);

		int lastVersion = 1;

		if (attProps.size() > 0) {
			// find last version of this file
			while (attProps.getProperty(lastVersion + ".author") != null) {
				lastVersion++;
			}
		}

		String[] extension = attName.split("\\.");
		String fileName = pathToAttachments
				+ "/"
				+ pageName
				+ "-att/"
				+ attName
				+ "-dir/"
				+ lastVersion
				+ "."
				+ ((attName.indexOf('.') != -1) ? extension[extension.length - 1]
						: "bin");

		InputStream oldFile = null;
		FileOutputStream newFile = null;

		try {
			newFile = new FileOutputStream(new File(fileName));
			oldFile = attFile;
		} catch (FileNotFoundException e) {
			log.error(e);
		}

		// writing old file to new one
		try {
			byte[] buffer = new byte[BUF_SIZE];
			int read;
			while ((read = oldFile.read(buffer)) != -1)
				newFile.write(buffer, 0, read);
		} catch (Exception e2) {
			// Nothing that we can really do. Just send an incomplete
			// file.
		} finally {
			try {
				newFile.close();
				oldFile.close();
			} catch (IOException e) {
				log.error(e);
			}
		}

		attProps.setProperty(lastVersion + ".author", user);

		saveAttProps(attProps, pageName, attName);
	}

	public Set<String> getAttachmentsSet(String uid) {
		Set<String> attSet = new HashSet<String>();

		File attDir = new File(pathToAttachments + "/" + uid + "-att");

		if (attDir.exists() && (attDir.listFiles().length > 0)) {
			File[] attachements = attDir.listFiles();

			for (int i = 0; i < attachements.length; i++) {
				if ((attachements[i].isDirectory())
						&& (attachements[i].getName().endsWith("-dir"))) {

					String[] tokens = attachements[i].getName().split("/");
					attSet.add(tokens[tokens.length - 1].substring(0,
							tokens[tokens.length - 1].length() - 4));

				}
			}
		} else {
			// page has no attachements
			return null;
		}

		return attSet;
	}
	
	public void setLanguageCode(String langCode) {
		init(langCode);

		this.langCode = langCode;
	}
	
	private Properties getAttProps(String pageName, String attName,
			boolean createProps) {
		Properties attProps = new Properties();

		File propFile = new File(pathToAttachments + "/" + pageName + "-att/"
				+ attName + "-dir/");

		if (!propFile.exists()) {

			if (createProps) {
				// create dirs for this property file
				propFile.mkdirs();
			} else {
				return null;
			}
		}

		propFile = new File(pathToAttachments + "/" + pageName + "-att/"
				+ attName + "-dir/attachment.properties");

		try {
			attProps.load(new FileInputStream(propFile));
		} catch (IOException ioe) {
			System.err
					.println("Cannot read attachment.properties for attachement: "
							+ pathToAttachments
							+ "/"
							+ pageName
							+ "-att/"
							+ attName + "-dir/attachment.properties\n" + ioe);
		}

		return attProps;
	}
	
	private void saveAttProps(Properties props, String pageName, String attName) {
		try {
			props.store(new FileOutputStream(new File(pathToAttachments + "/"
					+ pageName + "-att/" + attName
					+ "-dir/attachment.properties")), FileMediaDataSource.SAVE_COMMENT);
		} catch (Exception ioe) {
			System.err
					.println("Cannot store attchement.properties for attachement: "
							+ pathToAttachments
							+ "/"
							+ pageName
							+ "-att/"
							+ attName + "-dir/attachments.properties\n" + ioe);
		}

	}
}
