/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.jboss.logging.Logger;
import org.jboss.wiki.exceptions.PageRenamingException;

public class FileWikiPageDictionary implements WikiPageDictionary {

	private final static String OTHER_LANG_DIR = "LANG/";
	
	private Properties pageDictionary;

	private Properties pageRevDictionary;

	private File pageDictionaryFile;

	private boolean lock = false;
	
	private String langCode;

	private Logger log = Logger.getLogger(this.getClass());
	
	private String pathToMedia;

	public FileWikiPageDictionary() {
		init("");
	}

	private void init(String prefix) {
		log.info("Looking for props file !");

		Properties fileDSProps = new Properties();

		try {

			fileDSProps.load(FileMediaDataSource.class
					.getResourceAsStream(FileMediaDataSource.propFileName));

		} catch (IOException ioe) {
			log.error("Can't load the file " + FileMediaDataSource.propFileName + "\n", ioe);
		}

		loadProperties(fileDSProps, prefix);

		// load dictionary props
		pageDictionary = new Properties();
		pageRevDictionary = new Properties();

		// TODO Do not load dictionary YET !

		pageDictionaryFile = new File(pathToMedia + "/dictionary.properties");

		if (!pageDictionaryFile.exists()) {
			try {
				pageDictionaryFile.createNewFile();
			} catch (IOException e) {
				log.error("Cannot create dictionary file: ", e);
			}
		} else {
			// there is dictionary file. Load props from it
			FileInputStream is;
			try {
				is = new FileInputStream(pageDictionaryFile);

				try {
					pageDictionary.load(is);
				} catch (IOException e) {
					e.printStackTrace();
				}

				try {
					is.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}

		}

		// create pageRevDictionary - to make it work faster

		Set<Object> keys = pageDictionary.keySet();

		for (Object key : keys) {
			pageRevDictionary.setProperty(pageDictionary
					.getProperty((String) key), (String) key);
		}
	}

	public void loadProperties(Properties fileDSProps, String prefix) {
		if (fileDSProps.getProperty("pathToMedia") != null) {
			pathToMedia = fileDSProps.getProperty("pathToMedia")
					+ (prefix.length() > 0 ? "/" + prefix : "");
			log.info("Path to media is: " + pathToMedia);
			
			File mediaDir = new File(pathToMedia);
			
			if (!mediaDir.exists()) {
				mediaDir.mkdirs();
				log.info("Dir for attachments created");
			}
		} else {
			System.err
					.println("Your properties file lacks of pathToMedia property. It will not run properly ! Please update "
							+ FileMediaDataSource.propFileName);
		}
	}
	
	public Map<String, String> getRenameMapings() {
		Map<String, String> map = new HashMap<String, String>();

		for (Object uid : pageDictionary.keySet()) {
			map.put((String) uid, pageDictionary.getProperty((String) uid));
		}

		return map;
	}

	public String getRealName(String uid) {
		while (lock)
			; // wait till unlocked (dictionary is changing)

		return (pageDictionary.getProperty(uid) != null) ? pageDictionary
				.getProperty(uid) : uid;
	}

	public String getUid(String realName) {
		while (lock)
			; // wait till unlocked (dictionary is changing)

		return (pageRevDictionary.getProperty(realName) != null) ? pageRevDictionary
				.getProperty(realName)
				: realName;
	}

	public synchronized void rename(String uid, String newName)
			throws PageRenamingException {

		lock = true;

		pageDictionary.setProperty(uid, newName);
		pageRevDictionary.setProperty(newName, uid);

		saveDictionary();

		log.info("Renamed (uid) " + uid + " to " + newName);

		lock = false;
	}

	private void saveDictionary() {
		// synchronize dictionary with file on disk
		FileOutputStream fos;
		try {
			fos = new FileOutputStream(pageDictionaryFile);

			try {
				pageDictionary.store(fos, FileMediaDataSource.SAVE_COMMENT);
			} catch (IOException e) {
				e.printStackTrace();
			}

			try {
				fos.close();
			} catch (IOException e) {
				e.printStackTrace();
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}

	public boolean uidInDictionary(String uid) {
		return pageDictionary.contains(uid);
	}

	public boolean realNameInDictionary(String realName) {
		return pageRevDictionary.contains(realName);
	}

	public synchronized void removeDictForPage(String uid)
			throws PageRenamingException {
		if (!pageDictionary.containsKey(uid)) {
			throw new PageRenamingException(
					"There is no dictionary entry for uid: " + uid);
		}

		String realName = pageDictionary.getProperty(uid);

		// remove all dict entries
		pageDictionary.remove(uid);
		pageRevDictionary.remove(realName);

		saveDictionary();
	}

	public void setLanguageCode(String langCode) {
		init(OTHER_LANG_DIR + langCode);

		this.langCode = langCode;
	}

	public void setWikiEngine(WikiEngine wikiEngine) {
		
		if (langCode == null)
			langCode = wikiEngine.defaultLangugeCode;
	}

}
