/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.jboss.wiki.exceptions.DataSourceException;
import org.jboss.wiki.exceptions.WikiSaveException;

/**
 * @author <a href="mailto:tomasz.szymanski@jboss.com">Tomasz Szymanski</a>
 * 
 */
public interface MediaDataSource {

	public final static int EDITABLE = 100;

	public final static int VIEWABLE = 1;

	public static String mediaTrashName = "trash/pages";

	/**
	 * This function is executed before every save.
	 * 
	 * 
	 * @return true if preSafe function succeded. False if not.
	 */
	public void preSave() throws WikiSaveException;

	/**
	 * Saves given given wikiPage with language code
	 * 
	 * <b>please note languages are still not supported</b>
	 * 
	 * 
	 * @param page
	 *            Page to save (with changed version number, author and content)
	 * @param uid
	 *            Page's uid
	 * @return true if page was succesfully saved
	 */
	public void savePage(String uid, WikiPage page) throws WikiSaveException;

	/**
	 * This function is exeuted after saving page.
	 * 
	 * @return true if succeded. False otherwise.
	 */
	public void postSave() throws WikiSaveException;

	/**
	 * This function is executed before every page get.
	 * 
	 * 
	 * @return true if succeded. False otherwise.
	 */
	public boolean preGet();

	/**
	 * Gets page with given page name.
	 * 
	 * 
	 * @return true if succeded. False otherwise.
	 * @param pageName
	 *            Name of the page to get.
	 */
	public WikiPage getPage(String pageName);

	/**
	 * This function is executed after every page get.
	 * 
	 * 
	 * @return Desired wiki page or null if it doesn't exists.
	 */
	public boolean postGet();

	/**
	 * Gets page at given version and with given language code.
	 * 
	 * @param originPage
	 *            Original page to get version for.
	 * @param loadContent
	 *            If true returned page will be loaded with content.
	 * @param version
	 *            Version of the page to load.
	 * @return Desired wiki page or null if it doesn't exists. Page will be
	 *         loaded with content or not due to loadContent.
	 */
	public WikiPage getPageAtVersion(WikiPage originPage, String uid,
			boolean loadContent, int version);

	/**
	 * Checks if given page exists.
	 * 
	 * @param pageName
	 *            Name of the page you are looking for.
	 * @return true if page exists. False otherwise.
	 */
	public boolean pageExists(String pageName);

	/**
	 * Loads content to given page at given version.
	 * 
	 * @param page
	 *            Page to load content to.
	 * @param loadContent
	 *            If true it will load content. If false it will do nothing.
	 * @param version
	 *            Version of the page to load.
	 */
	public void getContentAtVersion(WikiPage page, String uid,
			boolean loadContent, int version);

	/**
	 * Gets all page names.
	 * 
	 * @return Set of page names.
	 */
	public Set<String> getAllPageNames();

	/**
	 * Gets subpages for a given page.
	 * 
	 * @param pageName
	 *            Page name to get subpages list for.
	 * @return List of subpages for a page pageName
	 */
	public Set<String> getPagesFor(String pageName);

	/**
	 * Gets rights (editable and/or viewable) for a given page. Result is sum of
	 * apropriate static variables EDITABLE and VIEWABLE
	 * 
	 * @param pageName
	 *            Name of the page to get rights for.
	 * @return Sum of apropriate static variables EDITABLE and VIEWABLE
	 */
	public int getPageMod(String pageName);

	/**
	 * Set right for a page.
	 * 
	 * @param pageName
	 *            Name of the page to set right for.
	 * @param mods
	 *            Sum of apropriate static variables EDITABLE and VIEWABLE
	 */
	public void setPageMod(String pageName, int mods);

	/**
	 * Deletes (or moves to trash) given page
	 * 
	 * @param pageName
	 *            Name of the page to delete.
	 */
	public boolean deletePage(String pageName);

	/**
	 * Sets wikiEngine reference
	 * 
	 * @param wikiEngine
	 */
	public void setWikiEngine(WikiEngine wikiEngine);

	/**
	 * Saves roles for given page.
	 * 
	 * @param uid
	 *            Page uid to save roles to
	 * @param roles
	 *            Roles to save
	 * @param action
	 *            can be MediaDataSource.EDITABLE or MediaDataSource.VIEWABLE
	 */
	public void saveRolesSet(String uid, Set<String> roles, int action)
			throws DataSourceException;

	/**
	 * Save metadata properties for given page.
	 * 
	 * @param uid
	 *            Page uid to save properties to.
	 */
	public void saveMetadataProps(String uid, Properties props);

	/**
	 * Gets metadata properties for given page.
	 * 
	 * @param uid
	 *            Page uid to get properties for.
	 * @return
	 */
	public Properties getMetadataProps(String uid);

	/**
	 * Set language code for current dataSource.
	 * 
	 * @param langCode
	 *            code of the language
	 */
	public void setLanguageCode(String langCode);

	/**
	 * Gets all available languages in this wiki. This should be only called to
	 * default DataSource
	 * 
	 * @return Map of sorted language two-letter codes and their descriptions
	 */
	public Map<String, String> getAvailableLanguages();

	/**
	 * Adds langCode (to descriptor). This should be only called to default
	 * DataSource
	 * 
	 * @param langCode
	 *            Two-letter language code
	 * @param langDesc
	 *            Language description
	 */
	public void addLangCode(String langCode, String langDesc);

	/**
	 * Get's language code from this datasource
	 * 
	 * @return two-letter language code
	 */
	public String getLangCode();

	/**
	 * Sets global property
	 * 
	 * @param key
	 *            Key
	 * @param value
	 *            Value
	 */
	public void setGlobalProperty(String key, String value) throws DataSourceException;

	/**
	 * Gets global property
	 * 
	 * @param key
	 *            Key
	 * @return Value
	 */
	public String getGlobalProperty(String key) throws DataSourceException;
}
