/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.InputStream;
import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import org.jboss.logging.Logger;

import org.jboss.wiki.exceptions.DataSourceException;
import org.jboss.wiki.exceptions.EditSessionExpired;
import org.jboss.wiki.exceptions.EditingNotAllowedException;
import org.jboss.wiki.exceptions.PageNotEditedException;
import org.jboss.wiki.exceptions.WikiSaveException;

/**
 * <p>
 * </p>
 * 
 */
/**
 * 
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a>
 * @author <a href="mailto:tomasz.szymanski@jboss.com">Tomasz Szymanski</a>
 * @version $Id$
 */
public class WikiPage implements Serializable, WikiProperties, Cloneable {

	public static final String OTHER_LANGS = "OTHER_LANGS";

	public static final String METADATA = "MetaData:Properties";

	private static final String LANG = "PAGE_LANG";

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/* number of minutes after which page lock will expire */
	private static final int LOCK_TIME = 30;

	private Date lockTime = null;

	private String lockUser;

	private long length = 0;

	private Set<Integer> expiredSessionIds;

	private HashMap<Integer, EditSessionWatcher> editSessionWatchers;

	/**
	 * Temp variables (cleaned after every process ends)
	 */
	private HashMap<String, Object> tempVariables;

	/**
	 * Permanent variables - lives untiles JBoss Wiki is up
	 */
	private HashMap<String, Object> permVariables;

	private String name;

	private Logger log;

	private Credentials lastAuthor;

	private int lastVersion;

	private int version;

	private Date editDate;

	private WikiEngine wikiEngine;

	public WikiEngine wikiEngine_1;

	private String pageContent;

	private boolean editable;

	private boolean viewable;

	private Set<String> editRoles;

	private Set<String> viewRoles;

	public WikiPage(String pageName, Credentials pageAuthor,
			String pageContent, int pageLastVersion, int thisVersion,
			Date editDate, WikiEngine wikiEngine, boolean viewable,
			boolean editable, Set<String> editRoles, Set<String> viewRoles,
			long length, String langCode) {

		tempVariables = new HashMap<String, Object>();
		permVariables = new HashMap<String, Object>();

		name = pageName;
		lastAuthor = pageAuthor;
		this.pageContent = pageContent;
		lastVersion = pageLastVersion;
		version = thisVersion;
		this.editDate = editDate;
		this.wikiEngine = wikiEngine;
		this.viewable = viewable;
		this.editable = editable;
		expiredSessionIds = new HashSet<Integer>();
		editSessionWatchers = new HashMap<Integer, EditSessionWatcher>();
		log = Logger.getLogger(WikiPage.class);

		this.editRoles = (editRoles == null) ? new TreeSet<String>()
				: editRoles;
		this.viewRoles = (viewRoles == null) ? new TreeSet<String>()
				: viewRoles;

		this.length = length;

		setLangCode(langCode);

	}

	public synchronized void save(Credentials credentials,
			Integer editActionId, String langCode, WikiSession wikiSession) throws EditSessionExpired,
			WikiSaveException, EditingNotAllowedException {

		if (expiredSessionIds.contains(editActionId)) {
			throw new EditSessionExpired(
					"Your edit session have expired. Please edit page once again !");
		}

		if (editSessionWatchers.get(editActionId) != null) {
			editSessionWatchers.get(editActionId).stopWatcher();
			editSessionWatchers.remove(editActionId);
		}

		wikiEngine.savePage(credentials, this, langCode, wikiSession);
	}

	public WikiPage getPageAtVersion(boolean loadContent, int version) {
		if (version == lastVersion) {
			return this;
		} else {
			return wikiEngine.getPageAtVersion(this, loadContent, version,
					getLangCode());
		}
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 */
	public String getContent() {
		return getPageContent();
	}

	public void showPage() {
		log.info("Page name: " + getName());
		log.info("Page author: " + getLastAuthor());
		log.info("Page version: " + getLastVersion());
		log.info("Page editdate: " + getEditDate());
		log.info("Page Content: \n" + getContent());
	}

	/**
	 * <p>
	 * </p>
	 * 
	 */
	public MediaInternalization mediaInternalization;

	public java.util.Date getEditDate() {
		return editDate;
	}

	public void setEditDate(java.util.Date editDate) {
		this.editDate = editDate;
	}

	public Credentials getLastAuthor() {
		return lastAuthor;
	}

	public void setLastAuthor(Credentials lastAuthor) {
		this.lastAuthor = lastAuthor;
	}

	public int getLastVersion() {
		return lastVersion;
	}

	public void setLastVersion(int lastVersion) {
		this.lastVersion = lastVersion;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getPageContent() {
		if (pageContent == null) {
			// page wasn't loaded with conent - lets do it
			wikiEngine.getContentAtVersion(this, true, version, getLangCode());
		}
		return pageContent;
	}

	public void setContent(String pageContent) {
		this.pageContent = pageContent;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		WikiPage clonedPage = new WikiPage(getName(), getLastAuthor(),
				getPageContent(), getLastVersion(), getVersion(),
				getEditDate(), wikiEngine, viewable, editable, editRoles,
				viewRoles, length, getLangCode());

		// copy tempVariables as they are temp only...
		clonedPage.setTempVariables(new HashMap<String, Object>(tempVariables));

		clonedPage.setPermVariables(permVariables);

		return clonedPage;
	}

	public void unlock(Credentials user, Integer editSessionId) {
		log.debug("UNLOCK: " + user.getName() + " save user: " + lockUser);

		if (editSessionWatchers.get(editSessionId) != null) {
			editSessionWatchers.get(editSessionId).stopWatcher();
			editSessionWatchers.remove(editSessionId);
		}

		try {
			wikiEngine.endEditing(editSessionId);
		} catch (PageNotEditedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if ((user.getName()).equals(lockUser)) {
			log.debug("Unlocking page");
			lockTime = null;
			lockUser = null;
		}
	}

	public Date lock(Credentials user, Integer editSessionId) {
		Date now = new Date();
		Date expDate = null;
		Date retDate = lockTime;

		if (this.lockTime != null) {
			// lets calculete expire date of current lock

			Calendar calExpireDate = Calendar.getInstance();
			calExpireDate.setTime(this.lockTime);
			calExpireDate.add(Calendar.MINUTE, LOCK_TIME);
			expDate = calExpireDate.getTime();

			if (expDate.before(now)) {
				// lock expired. take it out
				lockTime = null;
				lockUser = null;
			}
		}

		if (this.lockTime == null) {
			log.debug("LOCK: " + user.getName());

			// page isn't locked or lock expired. Lets lock it.
			this.lockTime = now;
			this.lockUser = user.getName();
			log.debug("LOCK: " + lockUser);
			retDate = null;
		}

		// Add EditSessionWatcher
		wikiEngine.addEditing(editSessionId);
		EditSessionWatcher eSW = new EditSessionWatcher(editSessionId,
				wikiEngine.getSessionIdExpireTime(), this);
		eSW.start();

		editSessionWatchers.put(editSessionId, eSW);

		return retDate;
	}

	public String getLockUser() {
		return lockUser;
	}

	public int getVersion() {
		return version;
	}

	public void setVersion(int version) {
		this.version = version;
	}

	public long getLength() {
		if ((length == 0) && (getPageContent().length() != 0)) {
			length = getPageContent().length();
		}

		return length;
	}

	public void setLength(long length) {
		this.length = length;
	}

	public void addAttachement(InputStream attFile, String attName, String user) {
		wikiEngine.addAttachment(attFile, attName, this, user, getLangCode());
	}

	public Set<String> getAttachementsSet() {
		return wikiEngine.getAttachmentsSet(this, getLangCode());
	}

	public boolean isEditable(Credentials user) {
		log.debug("Editable: " + user.isAdmin() + " " + editable + " "
				+ checkRoles(user, editRoles));

		return user.isAdmin() || (editable && checkRoles(user, editRoles));
	}

	public void setEditable(boolean editable) {
		this.editable = editable;

		wikiEngine.setPageMod(getName(), ((viewable) ? MediaDataSource.VIEWABLE
				: 0)
				+ ((editable) ? MediaDataSource.EDITABLE : 0), getLangCode());
	}

	public boolean isViewable(Credentials user) {

		log.debug("Viewable: " + user.isAdmin() + " " + viewable + " "
				+ checkRoles(user, viewRoles));

		return user.isAdmin() || (viewable && checkRoles(user, viewRoles));
	}

	public boolean isViewableForAll() {
		return viewable;
	}

	public boolean isEditableForAll() {
		return editable;
	}

	public void setViewable(boolean viewable) {
		this.viewable = viewable;

		wikiEngine.setPageMod(getName(), ((viewable) ? MediaDataSource.VIEWABLE
				: 0)
				+ ((editable) ? MediaDataSource.EDITABLE : 0), getLangCode());
	}

	public void addExpiredSessionId(Integer id) throws PageNotEditedException {
		expiredSessionIds.add(id);
		wikiEngine.endEditing(id);
	}

	public void addTempVariable(String key, Object value,
			WikiSession wikiSession) {
		tempVariables.put(wikiSession.getId() + key, value);
	}

	public Object getTempVariable(String key, WikiSession wikiSession) {
		return tempVariables.get(wikiSession.getId() + key);
	}

	public void removeTempVariable(String key, WikiSession wikiSession) {
		tempVariables.remove(wikiSession.getId() + key);
	}

	public void clearTempVariables(WikiSession wikiSession) {
		for (String key : tempVariables.keySet()) {
			if (key.startsWith(wikiSession.getId())) {
				tempVariables.remove(key);
			}
		}
	}

	public void addPermVariable(String key, Object value) {
		// TODO - cannot add METADATA property - tho must be thrown an Exception
		// or sth.
		if (!key.equals(WikiPage.METADATA)) {
			permVariables.put(key, value);
		}
	}

	public Object getPermVariable(String key) {
		return permVariables.get(key);
	}

	public void removePermVariable(String key) {
		permVariables.remove(key);
	}

	public void setEditRoles(Set<String> editRoles) {
		if (editRoles == null) {
			this.editRoles.clear();
		} else {
			this.editRoles = editRoles;
		}
	}

	public void setViewRoles(Set<String> viewRoles) {
		if (viewRoles == null) {
			this.viewRoles.clear();
		} else {
			this.viewRoles = viewRoles;
		}
	}

	public boolean containsEditRole(String role) {
		return editRoles.contains(role);
	}

	public boolean containsViewRole(String role) {
		return viewRoles.contains(role);
	}

	private boolean checkRoles(Credentials user, Set<String> roleSet) {
		// if roles are empty - return true
		if (roleSet == null || roleSet.isEmpty()) {
			return true;
		}

		// check if user is in any role
		for (String role : roleSet) {
			if (user.isUserInRole(role)) {
				return true;
			}
		}

		// nothing found - return false
		return false;
	}

	public void switchEditRole(String role) {
		if (editRoles.contains(role)) {
			editRoles.remove(role);
		} else
			editRoles.add(role);

		log.info("EDIT ROLES " + editRoles.isEmpty());

		try {
			wikiEngine.saveRolesSet(this, editRoles, MediaDataSource.EDITABLE,
					getLangCode());
		} catch (DataSourceException e) {
			log.error("Couldn't save edit roles for page: " + getName(), e);
		}
	}

	public void switchViewRole(String role) {
		if (viewRoles.contains(role)) {
			viewRoles.remove(role);
		} else
			viewRoles.add(role);

		log.info("VIEW ROLES " + viewRoles.isEmpty());

		try {
			wikiEngine.saveRolesSet(this, viewRoles, MediaDataSource.VIEWABLE,
					getLangCode());
		} catch (DataSourceException e) {
			log.error("Couldn't save view roles for page: " + getName(), e);
		}

	}

	public void setPermVariables(HashMap<String, Object> permVariables) {
		this.permVariables = permVariables;
	}

	public void setTempVariables(HashMap<String, Object> tempVariables) {
		this.tempVariables = tempVariables;
	}

	/**
	 * Has to be executed explicitely - so user knows he IS changing metaProps
	 * 
	 * @param metaProps
	 */
	public void setMetaDataProps(Properties metaProps) {
		permVariables.put(METADATA, metaProps);
	}

	public void setMetaDataProperty(String key, String value) {
		Properties props = null;

		if (permVariables.get(METADATA) != null) {
			props = (Properties) permVariables.get(METADATA);
		} else {
			props = new Properties();
		}

		props.setProperty(key, value);

		wikiEngine.saveMetaProps(props, this);

		wikiEngine.refreshPage(getName(), getLangCode());
	}

	public void resetMetaProps() {
		if (permVariables.get(METADATA) != null) {
			permVariables.remove(METADATA);
		}
	}

	public String getMetaDataProperty(String key) {
		if (permVariables.get(METADATA) != null) {
			return ((Properties) permVariables.get(METADATA)).getProperty(key);
		} else {
			return null;
		}
	}

	public String getLangCode() {
		return (String) permVariables.get(LANG);
	}

	public void setLangCode(String langCode) {
		permVariables.put(LANG, langCode);
	}

	public void removeMetaDataProperty(String key) {
		Properties props = null;

		if (permVariables.get(METADATA) != null) {
			props = (Properties) permVariables.get(METADATA);
		} else {
			props = new Properties();
		}

		props.remove(key);

		wikiEngine.saveMetaProps(props, this);
	}
}
