/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashSet;
import java.util.Set;

import org.jboss.wiki.WikiAttachment;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.exceptions.WikiException;

public class AttachmentTest extends WikiTest {

	private static final String ATT_CONTENT = "This is simple text attachment";

	private static final String ATT_NAME = "testFile.txt";

	private static final String ATT_CONTENT_2 = "This is simple text attachment, but second one";

	private static final String ATT_NAME_2 = "testAnotherFile.txt";

	public void testSaveGetDelete() throws WikiException, IOException {
		String testPage = createTestPage(plainUser, PAGE_CONTENT);

		WikiPage page = wikiEngine.getByName(testPage, null, langCode);

		File att = createTempFile(ATT_CONTENT);

		// add attachment
		page.addAttachement(new FileInputStream(att), ATT_NAME, plainUser
				.getName());

		att = createTempFile(ATT_CONTENT_2);

		// add attachment
		page.addAttachement(new FileInputStream(att), ATT_NAME_2, plainUser
				.getName());

		// reload page
		wikiEngine.refreshPage(testPage, langCode);

		page = wikiEngine.getByName(testPage, null, langCode);

		Set<String> attSet = page.getAttachementsSet();

		// see if attSet has two attachemnt, we just added
		Set<String> mySet = new HashSet<String>();
		mySet.add(ATT_NAME);
		mySet.add(ATT_NAME_2);

		assertEquals(attSet, mySet);

		// see if attachments contain our data
		WikiAttachment wikiAttachment1 = wikiEngine.getAttachment(testPage,
				ATT_NAME, langCode);
		WikiAttachment wikiAttachment2 = wikiEngine.getAttachment(testPage,
				ATT_NAME_2, langCode);

		assertEquals(
				loadContentFromAttachment(wikiAttachment1.getInputStream()),
				ATT_CONTENT);

		// load content for 2nd attachment

		assertEquals(
				loadContentFromAttachment(wikiAttachment2.getInputStream()),
				ATT_CONTENT_2);

		// delete attachments
		wikiEngine.deleteAttachment(testPage, ATT_NAME, langCode);
		wikiEngine.deleteAttachment(testPage, ATT_NAME_2, langCode);

		// check atts for page
		assertTrue(page.getAttachementsSet() == null);
	}

	public void testAttachmentVersions() throws WikiException, IOException {
		String testPage = createTestPage(plainUser, PAGE_CONTENT);

		WikiPage page = wikiEngine.getByName(testPage, null, langCode);

		File att = createTempFile(ATT_CONTENT);

		// add attachment
		page.addAttachement(new FileInputStream(att), ATT_NAME, plainUser
				.getName());

		File att2 = createTempFile(ATT_CONTENT_2);

		// add attachment with same name and different content
		page.addAttachement(new FileInputStream(att2), ATT_NAME, plainUser
				.getName());

		
		// get attachment
		WikiAttachment wikiAttachment = wikiEngine.getAttachment(testPage,
				ATT_NAME, langCode);

		// version should be "2"
		assertEquals(wikiAttachment.getVersion(), 2);

		// test att size
		assertEquals(wikiEngine.getAttachmentSize(testPage, ATT_NAME, 1, langCode), att.length());
		assertEquals(wikiEngine.getAttachmentSize(testPage, ATT_NAME, 2, langCode), att2.length());
		
		// check content of last attachment
		assertEquals(loadContentFromAttachment(wikiAttachment.getInputStream()), ATT_CONTENT_2);
		
		//load att at version 1
		wikiAttachment = wikiEngine.getAttachment(testPage, ATT_NAME, 1, langCode);
		
		// version should be "1"
		assertEquals(wikiAttachment.getVersion(), 1);

		// check content of last attachment
		assertEquals(loadContentFromAttachment(wikiAttachment.getInputStream()), ATT_CONTENT);
		
	}
	
	public void testDeleteMultipleAttachments() throws IOException, WikiException {
		String testPage = createTestPage(plainUser, PAGE_CONTENT);

		WikiPage page = wikiEngine.getByName(testPage, null, langCode);

		File att = createTempFile(ATT_CONTENT);

		// add attachment
		page.addAttachement(new FileInputStream(att), ATT_NAME, plainUser
				.getName());

		File att2 = createTempFile(ATT_CONTENT_2);

		// add attachment with same name and different content
		page.addAttachement(new FileInputStream(att2), ATT_NAME_2, plainUser
				.getName());
		
		// see if attSet has two attachemnt, we just added
		Set<String> mySet = new HashSet<String>();
		mySet.add(ATT_NAME);
		mySet.add(ATT_NAME_2);

		assertEquals(page.getAttachementsSet(), mySet);
		
		// delete all atts
		wikiEngine.deleteAttachments(testPage, langCode);
		
		// there should be no atts for page
		assertTrue(page.getAttachementsSet() == null);
	}

	private String loadContentFromAttachment(InputStream is) throws IOException {
		int chr;
		String content = "";

		while ((chr = is.read()) != -1) {
			content += (char) chr;
		}

		is.close();

		return content;
	}
}
