/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.test;

import java.util.Map;

import org.jboss.wiki.WikiContext;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.exceptions.WikiException;
import org.jboss.wiki.plugins.ReTranslateUids;

public class DictionaryTest extends WikiTest {
	
	private static final String PAGE_W_LINK_PREF = "This page contains single [";
	
	private static final String PAGE_W_LINK_SUFF = "] - that is very linky";
	
	private static final String LINK1 = "TestLink";
	
	private static final String LINK2 = "AnotherLink";
	
	public void testDictionary() throws WikiException, InterruptedException{
		
		WikiContext wikiContext = new WikiContext(plainUser, wikiEngine.getWikiType("html"), testWikiSession);
		
		String page1 = createTestPage(plainUser, "1 page number 1");
		String page2 = createTestPage(plainUser, "2 page number 2");
		
		WikiPage wikiPage1 = wikiEngine.getByName(page1, null, langCode);
		WikiPage wikiPage2 = wikiEngine.getByName(page2, null, langCode);
		
		// set apropriate content of pages
		wikiPage1.setContent("1 ["+page2+"]");
		wikiPage1.setLastVersion(wikiPage1.getLastVersion()+1);
		wikiPage1.save(plainUser, wikiEngine.getEditSessionId(), langCode, testWikiSession);
		
		wikiPage2.setContent("1 ["+page2+"]");
		wikiPage2.setLastVersion(wikiPage2.getLastVersion()+1);
		wikiPage2.save(plainUser, wikiEngine.getEditSessionId(), langCode, testWikiSession);
		
		int i = 0;
		String newPage2Name = page2 + (i++);
		
		while(wikiEngine.pageExists(newPage2Name, langCode)) {
			newPage2Name = page2 + (i++);
		}
		
		log.info("New name for page2: "+newPage2Name);
		
		addRenaming(page2, newPage2Name);
		
		// wait for page to rename
		while (!wikiEngine.getPagesWaiting(langCode).isEmpty()) {
			Thread.sleep(5 * 1000);
		}
		
		Map<String,String> pageMapings = wikiEngine.getRenameMapings(langCode);
		for (String uid : pageMapings.keySet()) {
			System.out.println("Uid: "+uid+" Real: "+pageMapings.get(uid));
		}
		
		// add new page to deleting
		pagesToDelete.remove(page2);
		pagesToDelete.add(newPage2Name);
		
		// see if page after translation exists
		assertTrue(wikiEngine.pageExists(newPage2Name, langCode));
		
		// get page with renamed reference
		WikiPage afterRename = wikiEngine.getByName(page1, wikiContext, langCode);
		
		String pageContentRenamed = afterRename.getContent();
		
		log.info("page1 content: "+pageContentRenamed);
		
		int link = -1;
		
		// look if there is any link
		if ((link = pageContentRenamed.lastIndexOf(hostURL+"/wiki/")) == -1) {
			fail("There is no link in afterRename page.");
		}
		else {
			// find link in page
			String pageName = pageContentRenamed.substring(link + (hostURL+"/wiki/").length(), pageContentRenamed.indexOf("\"", link + (hostURL+"/wiki/").length()));
			log.info("PAGENAME: "+pageName);
			
			
			// check if the link is the one renamed in dictionary
			assertEquals(pageName, newPage2Name);
			
		}
	}
	
	public void testTranslating () throws WikiException {
		WikiContext wikiContext = new WikiContext(plainUser, wikiEngine.getWikiType("edit"), testWikiSession);
		
		// create first uniqueLink
		String uniqueLink = getUniqueName(LINK1);
		
		// create another uniqueLink, we'll make translation to
		String uniqueLinkTo = getUniqueName(LINK2);
		
		// create page with first link
		String testPage = createTestPage(plainUser, PAGE_W_LINK_PREF + uniqueLink + PAGE_W_LINK_SUFF);
		
		// add renaming
		addRenaming(uniqueLink, uniqueLinkTo);
		
		// sourcePage - will have link to first uniqueLink
		WikiPage sourcePage = wikiEngine.getByName(testPage, null, langCode);
		
		//page after translation - will have link to second uniqueLink
		WikiPage page = wikiEngine.getByName(testPage, wikiContext, langCode);

		// source page should have link to 1st uniqueLink
		assertTrue(sourcePage.getContent().indexOf(uniqueLink) != -1);
		
		// translated page should have link to 2nd uniqueLink
		assertTrue(page.getContent().indexOf(uniqueLinkTo) != -1);
		
		// page content should differ
		assertFalse(page.getContent().equals(sourcePage.getContent()));
		
		// retranslate uids 
		ReTranslateUids reTranslateUids = new ReTranslateUids(wikiEngine);
		
		// get retranslated content
		String reTranslated = reTranslateUids.getNewContent(page.getContent(), langCode);
		
		// retranslated should be the same as sourcePage
		assertEquals(reTranslated, sourcePage.getContent());
	}
}
