/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.test;

import java.util.Date;
import java.util.Set;
import java.util.Vector;

import org.jboss.wiki.WikiContext;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.exceptions.WikiException;

public class GetSavePageTest extends WikiTest {

	private final static String PAGE_CONTENT = "This is simple conten link Hello !";

	private final static String PAGE_CONTENT_2 = "This is another content";

	private final static String PAGE_VER_NAME = "TestVersionsPage";
	
	private final static int NUM_OF_SESSIDS = 100;

	public void testSaveGetPage() throws WikiException {
		WikiContext wikiContext = new WikiContext(plainUser, wikiEngine
				.getWikiType("html"), testWikiSession);

		String testPageName = createTestPage(plainUser, PAGE_CONTENT);

		assertTrue(wikiEngine.pageExists(testPageName, langCode));

		WikiPage wikiPage2 = wikiEngine.getByName(testPageName, wikiContext,
				langCode);

		String pageContent = wikiPage2.getPageContent();

		assertTrue(pageContent.indexOf(PAGE_CONTENT) != -1);

	}

	public void testGetAllPages() throws WikiException {
		Set<String> pageNames = wikiEngine.getAllPageNames(langCode);

		for (String pageName : pageNames) {
			assertTrue(wikiEngine.pageExists(pageName, langCode));
		}
		
		String newPage = createTestPage(plainUser, PAGE_CONTENT);
		
		Set<String> newPageNames = wikiEngine.getAllPageNames(langCode);
		
		pageNames.add(newPage);
		
		// check if the previous set is was just set after adding a page - this page
		assertEquals(pageNames, newPageNames);
	}

	public void testLocking() throws WikiException {
		WikiPage wikiPage = wikiEngine.getByName(createTestPage(plainUser,
				"some page content"), null, langCode);

		Vector<Integer> sessIds = new Vector<Integer>();

		// genrate sess ids and lock pages
		for (int i = 0; i < NUM_OF_SESSIDS; i++) {
			sessIds.add(wikiEngine.getEditSessionId());

			// check if lock gives a Date
			if (i > 0) {
				assertTrue(wikiPage.lock(plainUser, sessIds.get(i)) != null);
			} else {
				assertTrue(wikiPage.lock(plainUser, sessIds.get(i)) == null);
			}
		}
		
		// try saving page with wrong sessionId
		wikiPage.save(plainUser, NUM_OF_SESSIDS * 2, langCode, testWikiSession);

		// check what wikiEngine thinks
		assertTrue(wikiEngine.arePagesEdited());

		// unlock
		for (int i = 0; i < NUM_OF_SESSIDS; i++) {
			wikiPage.unlock(plainUser, sessIds.get(i));
		}

		// check what wikiengine thinks
		assertFalse(wikiEngine.arePagesEdited());
	}

	public void testVersions() throws WikiException {
		String testPageName = getUniqueName(PAGE_VER_NAME);

		// create page at version 1
		WikiPage testPage = new WikiPage(testPageName, plainUser, PAGE_CONTENT,
				1, 1, new Date(), wikiEngine, true, true, null, null,
				PAGE_CONTENT.length(), langCode);
		
		// save page
		wikiEngine.savePage(plainUser, testPage, langCode, testWikiSession);
		
		// add testpage to to-delete list
		pagesToDelete.add(testPageName);
		
		// check if it exists
		assertTrue(wikiEngine.pageExists(testPageName, langCode));
		
		// get the page again
		testPage = wikiEngine.getByName(testPageName, null, langCode);
		
		// save next version
		testPage.setContent(PAGE_CONTENT_2);
		testPage.setLastVersion(testPage.getLastVersion() + 1);
		
		wikiEngine.savePage(plainUser, testPage, langCode, testWikiSession);
		
		// get page again at current version
		testPage = wikiEngine.getByName(testPageName, null, langCode);
		
		// check content and version
		assertEquals(testPage.getContent(), PAGE_CONTENT_2);
		assertEquals(testPage.getVersion(), 2);
		
		// and at version 1
		WikiPage prevVersionPage = wikiEngine.getByName(testPageName, null, 1, langCode);
		
		// check content and version
		assertEquals(prevVersionPage.getContent(), PAGE_CONTENT);
		assertEquals(prevVersionPage.getVersion(), 1);
	}
}
