/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.test;

import org.jboss.wiki.MediaDataSource;
import org.jboss.wiki.WikiContext;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.exceptions.EditingNotAllowedException;
import org.jboss.wiki.exceptions.WikiException;

public class PageModsTest extends WikiTest {

	private final static String PAGE_CONTENT = "This is a simple page content.";

	public void testGlobalMods() throws WikiException {
		String testPage = createTestPage(plainUser, PAGE_CONTENT);

		WikiPage page = wikiEngine.getByName(testPage, null, langCode);

		// no mods - version is 1
		assertEquals(page.getLastVersion(), 1);

		// set page not editable
		page.setEditable(false);

		assertEquals(wikiEngine.getPageMod(testPage, langCode),
				MediaDataSource.VIEWABLE);
		
		// try to save page - it should throw an error
		page.setContent(PAGE_CONTENT + PAGE_CONTENT);
		page.setLastVersion(page.getLastVersion() + 1);
		
		try {
			wikiEngine.savePage(plainUser, page, langCode, testWikiSession);
			fail("Saving should throw an exception - page isn't editable");
		} catch (EditingNotAllowedException e) {
			// it's ok - page isn't editable
		}

		// set page not viewable
		page.setViewable(false);

		// try to get the page
		page = wikiEngine.getByName(testPage, new WikiContext(notLogedIn,
				wikiEngine.getWikiType("html"), testWikiSession), langCode);

		// if page wasn't viewable - it's version will be 0
		assertEquals(page.getVersion(), 0);

		// try the same with getMods - if page is not viewable and not editable - mod = 0
		assertEquals(wikiEngine.getPageMod(testPage, langCode), 0);
		
		// check if admin has access
		page = wikiEngine.getByName(testPage, new WikiContext(adminUser,
				wikiEngine.getWikiType("html"), testWikiSession), langCode);
		
		assertEquals(page.getVersion(), 1);

	}
	
	public void testRolesSet() throws WikiException {
		String testPage = createTestPage(plainUser, PAGE_CONTENT);

		WikiPage page = wikiEngine.getByName(testPage, null, langCode);
		
		page.setEditRoles(roles);
		wikiEngine.saveRolesSet(page, roles, MediaDataSource.EDITABLE, langCode);
		
		page.setViewRoles(roles);
		wikiEngine.saveRolesSet(page, roles, MediaDataSource.VIEWABLE, langCode);
		
		// reload the page from DS
		wikiEngine.refreshPage(testPage, langCode);
		
		// try to save page - it should throw an error
		page.setContent(PAGE_CONTENT + PAGE_CONTENT);
		page.setLastVersion(page.getLastVersion() + 1);
		
		try {
			wikiEngine.savePage(plainUser, page, langCode, testWikiSession);
			fail("Saving should throw an exception - page isn't editable");
		} catch (EditingNotAllowedException e) {
			// it's ok - page isn't editable
		}
		
		// try to get the page with not allowed user
		page = wikiEngine.getByName(testPage, new WikiContext(plainUser,
				wikiEngine.getWikiType("html"), testWikiSession), langCode);

		// if page wasn't viewable - it's version will be 0
		assertEquals(page.getVersion(), 0);
		
		// try to get the page with allowed user
		page = wikiEngine.getByName(testPage, new WikiContext(plainUserRoles,
				wikiEngine.getWikiType("html"), testWikiSession), langCode);

		// if page is viewable - it's version will be 1
		assertEquals(page.getVersion(), 1);
		
		// look if page is editable
		assertTrue(page.isEditable(plainUserRoles));
		assertFalse(page.isEditable(plainUser));
		
		// check if admin has access
		page = wikiEngine.getByName(testPage, new WikiContext(adminUser,
				wikiEngine.getWikiType("html"), testWikiSession), langCode);
		
		assertEquals(page.getVersion(), 1);

	}

}
