/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.test;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeSet;

import junit.framework.TestCase;

import org.apache.log4j.Logger;

import org.jboss.wiki.Credentials;
import org.jboss.wiki.WikiCommon;
import org.jboss.wiki.WikiContext;
import org.jboss.wiki.WikiEngine;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiSession;
import org.jboss.wiki.exceptions.PageRenamingException;
import org.jboss.wiki.exceptions.WikiException;
import org.jboss.wiki.exceptions.WikiManagementNotFoundException;
import org.jboss.wiki.management.WikiServiceManagement;

public class WikiTest extends TestCase {
	protected WikiEngine wikiEngine;

	protected WikiSession testWikiSession;

	protected Credentials notLogedIn = new TestCredentials(false, false);

	protected Credentials adminUser = new TestCredentials(true, true);

	protected Credentials plainUser = new TestCredentials(true, false);

	protected Credentials plainUserRoles;

	protected Set<String> pagesToDelete;

	protected Set<File> filesToDelete;

	protected Set<String> dictsToRemove;

	protected final static String TEST_PAGE_BASE_NAME = "TestPage";

	protected final static String hostURL = "http://localhost:8080";

	protected final static String actionURL = "/Szimano";

	protected final static String ROLE_NAME = "MyRole";

	protected final static String PAGE_CONTENT = "This is a simple page content.";

	protected final static String FILE_PREFIX = "wikiTestFile";

	protected final static String FILE_SUFFIX = ".wikiFile";

	protected Set<String> roles;

	protected String langCode;

	protected Logger log;

	@Override
	protected void setUp() throws Exception {
		log = Logger.getLogger(WikiTest.class);

		WikiCommon wikiCommon = new WikiCommon();

		try {
			WikiServiceManagement wikiServiceMenagement = wikiCommon
					.getWikiManagement();
			wikiEngine = wikiServiceMenagement.getWikiEngine();
		} catch (WikiManagementNotFoundException e1) {
			log.info("Couldn't find WikiManagement - probably u're executing tests outside AS");
			wikiEngine = new WikiEngine();
		}

		testWikiSession = new TestWikiSession();
		pagesToDelete = new HashSet<String>();
		filesToDelete = new HashSet<File>();
		dictsToRemove = new HashSet<String>();

		// set session atributes
		testWikiSession.setAttribute("credentials", plainUser);
		testWikiSession.setAttribute("actionURL", actionURL);
		testWikiSession.setAttribute("hostURL", hostURL);

		langCode = wikiEngine.defaultLangugeCode;

		roles = new TreeSet<String>();
		roles.add(ROLE_NAME);

		plainUserRoles = new TestCredentials(true, false, roles);
	}

	protected String createTestPage(Credentials user, String pageContent)
			throws WikiException {
		WikiContext wikiContext = new WikiContext(user, wikiEngine
				.getWikiType("html"), testWikiSession);

		if (wikiContext == null) {
			fail("There is no html wikitype");
		}

		String testPageName = TEST_PAGE_BASE_NAME;
		int i = 0;

		while (wikiEngine.pageExists(testPageName, langCode)) {
			testPageName = TEST_PAGE_BASE_NAME + (i++);
		}

		WikiPage wikiPage = new WikiPage(testPageName, user, pageContent, 1, 1,
				new Date(), wikiEngine, true, true, null, null, pageContent
						.length(), langCode);

		wikiPage.save(plainUser, wikiEngine.getEditSessionId(), langCode, testWikiSession);

		pagesToDelete.add(testPageName);

		return testPageName;
	}

	protected String getUniqueName(String prefix) {

		if (wikiEngine.pageExists(prefix, langCode)) {
			int i = 0;

			while (wikiEngine.pageExists(prefix + i, langCode)) {
				i++;
			}

			return prefix + i;
		}

		return prefix;
	}

	private void deleteAllTestPages() {
		boolean ret = true;

		for (String pageName : pagesToDelete) {
			if (!wikiEngine.deletePage(pageName, true, langCode)) {
				ret = false;
				log.error("Couldn't delete " + pageName);
			}
		}

		if (!ret) {
			// fail("Problems with deleting pages");
		}
	}

	@Override
	protected void tearDown() throws Exception {
		// at the end delete created testpages

		deleteAllTestPages();

		deleteAllTempFiles();

		removeDicts();

		super.tearDown();
	}

	private void removeDicts() throws PageRenamingException,
			InterruptedException {
		for (String dict : dictsToRemove) {
			wikiEngine.removeDictForPage(dict, langCode);
			System.out.println("Removed dict: "+dict);
		}

		// wait for all pages beeing deleted
		while (!wikiEngine.getDictRemovesWaiting(langCode).isEmpty()) {
			Thread.sleep(5 * 1000);
		}
	}

	private void deleteAllTempFiles() {
		for (File f : filesToDelete) {
			f.delete();
		}
	}

	protected File createTempFile(String content) throws IOException {
		File attFile = File.createTempFile(FILE_PREFIX, FILE_SUFFIX);

		FileOutputStream fos = new FileOutputStream(attFile);

		for (int i = 0; i < content.length(); i++) {
			fos.write(content.charAt(i));
		}

		fos.close();

		return attFile;
	}

	protected void addRenaming(String uid, String newName)
			throws PageRenamingException {
		wikiEngine.rename(uid, newName, langCode);
		
		System.out.println("Added renaming: "+uid+" -> "+newName);

		dictsToRemove.add(newName);
	}
}
